-- Test payment_account_bank
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(29);

set search_path to numerus, auth, public;

select has_table('payment_account_bank');
select has_pk('payment_account_bank');
select table_privs_are('payment_account_bank', 'guest', array []::text[]);
select table_privs_are('payment_account_bank', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_account_bank', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_account_bank', 'authenticator', array []::text[]);

select col_is_fk('payment_account_bank', array['payment_account_id', 'payment_account_type']);
select fk_ok('payment_account_bank', array['payment_account_id', 'payment_account_type'], 'payment_account', array['payment_account_id', 'payment_account_type']);

select has_column('payment_account_bank', 'payment_account_id');
select col_is_pk('payment_account_bank', 'payment_account_id');
select col_type_is('payment_account_bank', 'payment_account_id', 'integer');
select col_not_null('payment_account_bank', 'payment_account_id');
select col_hasnt_default('payment_account_bank', 'payment_account_id');

select has_column('payment_account_bank', 'payment_account_type');
select col_type_is('payment_account_bank', 'payment_account_type', 'text');
select col_not_null('payment_account_bank', 'payment_account_type');
select col_has_default('payment_account_bank', 'payment_account_type');
select col_default_is('payment_account_bank', 'payment_account_type', 'bank');

select has_column('payment_account_bank', 'iban');
select col_type_is('payment_account_bank', 'iban', 'iban');
select col_not_null('payment_account_bank', 'iban');
select col_hasnt_default('payment_account_bank', 'iban');


set client_min_messages to warning;
truncate payment_account_bank cascade;
truncate payment_account cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into payment_account (payment_account_id, company_id, payment_account_type, name)
values ( 6, 2, 'bank', 'Bank 2')
     , ( 7, 2, 'card', 'Credit card 2')
     , ( 8, 2, 'cash', 'Cash 2')
     , ( 9, 2, 'other', 'Other 2')
     , (10, 4, 'bank', 'Bank 4')
;

insert into payment_account_bank (payment_account_id, iban)
values ( 6, 'NL35INGB5262865534')
     , (10, 'MT47JQRS54557143744629565915326')
;

prepare payment_account_data as
select company_id, iban::text
from payment_account
join payment_account_bank using (payment_account_id)
order by payment_account_id, iban;

set role invoicer;
select is_empty('payment_account_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'payment_account_data',
	$$ values (2, 'NL35INGB5262865534')
	$$,
	'Should only list payment_accounts of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'payment_account_data',
	$$ values (4, 'MT47JQRS54557143744629565915326')
	$$,
	'Should only list payment_accounts of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'payment_account_data',
	'42501', 'permission denied for table payment_account',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into payment_account_bank (payment_account_id, payment_account_type, iban)
	values (7, 'card', 'ES2820958297603648596978')
	$$,
       '23514', 'new row for relation "payment_account_bank" violates check constraint "payment_account_type_is_bank"',
	'Should not allow payment accounts of type card'
);

select throws_ok( $$
	insert into payment_account_bank (payment_account_id, payment_account_type, iban)
	values (8, 'cash', 'ES2820958297603648596978')
	$$,
       '23514', 'new row for relation "payment_account_bank" violates check constraint "payment_account_type_is_bank"',
	'Should not allow payment accounts of type cash'
);

select throws_ok( $$
	insert into payment_account_bank (payment_account_id, payment_account_type, iban)
	values (9, 'other', 'ES2820958297603648596978')
	$$,
       '23514', 'new row for relation "payment_account_bank" violates check constraint "payment_account_type_is_bank"',
	'Should not allow payment accounts of type other'
);

select *
from finish();

rollback;

